import React, { useContext, useEffect, useRef, useState, Component } from 'react';
import { ParallaxController, ScrollAxis } from 'parallax-controller';

var ParallaxContext = /*#__PURE__*/React.createContext(null);

function useController() {
  var parallaxController = useContext(ParallaxContext);

  if (!parallaxController) {
    throw new Error('Could not find `react-scroll-parallax` context value. Please ensure the component is wrapped in a <ParallaxProvider>');
  }

  return parallaxController;
}

function removeUndefinedObjectKeys(obj) {
  Object.keys(obj).forEach(function (key) {
    return obj[key] === undefined ? delete obj[key] : {};
  });
  return obj;
}

function useVerifyController(controller) {
  useEffect(function () {
    // Make sure the provided controller is an instance of the Parallax Controller
    var isInstance = controller instanceof ParallaxController; // Throw if neither context or global is available

    if (!controller && !isInstance) {
      throw new Error("Must wrap your application's <Parallax /> components in a <ParallaxProvider />.");
    }
  }, [controller]);
}

function Parallax(props) {
  var controller = useController();
  var refInner = useRef();
  var refOuter = useRef();
  useVerifyController(controller);

  function _getElementOptions() {
    var useSpeedProp = typeof props.speed !== 'undefined';
    var isHorizontal = controller.scrollAxis == 'horizontal';
    var isVertical = controller.scrollAxis == 'vertical';
    var translateX = props.translateX;
    var translateY = props.translateY;

    if (useSpeedProp && isHorizontal) {
      translateX = [(props.speed || 0) * 10 + "px", (props.speed || 0) * -10 + "px"];
    }

    if (useSpeedProp && isVertical) {
      translateY = [(props.speed || 0) * 10 + "px", (props.speed || 0) * -10 + "px"];
    }

    return {
      elInner: refInner.current,
      elOuter: refOuter.current,
      props: removeUndefinedObjectKeys({
        disabled: props.disabled,
        translateX: translateX,
        translateY: translateY,
        rotate: props.rotate,
        rotateX: props.rotateX,
        rotateY: props.rotateY,
        rotateZ: props.rotateZ,
        scale: props.scale,
        scaleX: props.scaleX,
        scaleY: props.scaleY,
        scaleZ: props.scaleZ,
        opacity: props.opacity
      })
    };
  }

  var _useState = useState(),
      element = _useState[0],
      setElement = _useState[1]; // create element


  useEffect(function () {
    var newElement = controller.createElement(_getElementOptions());
    setElement(newElement);
    return function () {
      return controller.removeElementById(newElement.id);
    };
  }, []); // update element

  useEffect(function () {
    if (element) {
      if (props.disabled) {
        controller.resetElementStyles(element);
      } else {
        controller.updateElementPropsById(element.id, _getElementOptions().props);
      }
    }
  }, [props.disabled, props.translateX, props.translateY, props.rotate, props.rotateX, props.rotateY, props.rotateZ, props.scale, props.scaleX, props.scaleY, props.scaleZ, props.speed, props.opacity]);
  var Outer = props.tagOuter;
  var Inner = props.tagInner;
  var rootClass = 'parallax-outer' + (props.className ? " " + props.className : '');
  return React.createElement(Outer, {
    className: rootClass,
    ref: refOuter,
    style: props.styleOuter
  }, React.createElement(Inner, {
    className: "parallax-inner",
    ref: refInner,
    style: props.styleInner
  }, props.children));
}
Parallax.defaultProps = {
  disabled: false,
  tagInner: 'div',
  tagOuter: 'div'
};

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;

  _setPrototypeOf(subClass, superClass);
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };

  return _setPrototypeOf(o, p);
}

var containerStyle = {
  position: 'relative',
  overflow: 'hidden',
  width: '100%',
  height: '50vh'
};
var absoluteStyle = {
  position: 'absolute',
  top: 0,
  right: 0,
  bottom: 0,
  left: 0
};
var ParallaxBanner = function ParallaxBanner(_ref) {
  var children = _ref.children,
      className = _ref.className,
      layers = _ref.layers,
      style = _ref.style,
      disabled = _ref.disabled;
  return React.createElement("div", {
    style: _extends({}, containerStyle, style),
    className: 'parallax-banner' + (className ? " " + className : '')
  }, layers.map(function (_ref2, i) {
    var amount = _ref2.amount,
        layerChildren = _ref2.children,
        _ref2$expanded = _ref2.expanded,
        expanded = _ref2$expanded === void 0 ? true : _ref2$expanded,
        image = _ref2.image,
        _ref2$props = _ref2.props,
        props = _ref2$props === void 0 ? {} : _ref2$props;
    // save props to be merged
    var layerStyle = props.style || {};
    var layerClass = props.className || ''; // remove from pass through props

    delete props.style;
    delete props.className;
    var layerClassMerged = "parallax-banner-layer-" + i + (layerClass ? " " + layerClass : ''); // if this is an expanded layer overwrite the top/bottom styles with negative margins

    var expandedStyle = expanded ? {
      top: Math.abs(amount) * 100 * -1 + '%',
      bottom: Math.abs(amount) * 100 * -1 + '%'
    } : {}; // optional image styles

    var imageStyle = image ? {
      backgroundImage: "url(" + image + ")",
      backgroundPosition: 'center',
      backgroundSize: 'cover'
    } : {};
    return React.createElement(Parallax, {
      key: "layer-" + i,
      translateY: [amount * -1 * 100 + '%', amount * 100 + '%'],
      styleInner: absoluteStyle,
      styleOuter: absoluteStyle,
      disabled: disabled
    }, React.createElement("div", Object.assign({
      className: layerClassMerged,
      style: _extends({}, imageStyle, absoluteStyle, expandedStyle, layerStyle)
    }, props), layerChildren));
  }), children);
};
ParallaxBanner.defaultProps = {
  disabled: false
};

var createController = function createController(options) {
  // Don't initialize on the server
  var isServer = typeof window === 'undefined';

  if (!isServer) {
    // Must not be the server so kick it off...
    return ParallaxController.init(options);
  }

  return null;
};

var ParallaxProvider = /*#__PURE__*/function (_Component) {
  _inheritsLoose(ParallaxProvider, _Component);

  function ParallaxProvider(props) {
    var _this;

    _this = _Component.call(this, props) || this;
    _this.controller = createController({
      scrollAxis: props.scrollAxis,
      scrollContainer: props.scrollContainer
    });
    return _this;
  }

  var _proto = ParallaxProvider.prototype;

  _proto.componentDidUpdate = function componentDidUpdate(prevProps) {
    if (prevProps.scrollContainer !== this.props.scrollContainer) {
      // @ts-ignore
      this.controller.updateScrollContainer(this.props.scrollContainer);
    }
  };

  _proto.componentWillUnmount = function componentWillUnmount() {
    // @ts-ignore
    this.controller = this.controller.destroy();
  };

  _proto.render = function render() {
    var children = this.props.children;
    return (// @ts-ignore
      React.createElement(ParallaxContext.Provider, {
        value: this.controller
      }, children)
    );
  };

  return ParallaxProvider;
}(Component);
ParallaxProvider.defaultProps = {
  scrollAxis: ScrollAxis.vertical
};

export { Parallax, ParallaxBanner, ParallaxContext, ParallaxProvider, useController };
//# sourceMappingURL=react-scroll-parallax.esm.js.map
